#ifndef _CAMERA_H_
#define _CAMERA_H_

#include <cassert>
#include <iostream>
#include <fstream>
#include "vectors.h"

// ====================================================================

class Camera {

public:
  // CONSTRUCTOR & DESTRUCTOR
  Camera(const Vec3f &c, const Vec3f &poi, const Vec3f &u);
  virtual ~Camera() {}

  // GL NAVIGATION
  virtual void glInit(int w, int h) = 0;
  void glPlaceCamera(void);
  void dollyCamera(float dist);
  virtual void zoomCamera(float dist) = 0;
  void truckCamera(float dx, float dy);
  void rotateCamera(float rx, float ry);
  friend std::ostream& operator<<(std::ostream &ostr, const Camera &c);

protected:
  Camera() { assert(0); } // don't use

  // HELPER FUNCTIONS
  Vec3f getHorizontal() const {
    Vec3f answer;
    Vec3f::Cross3(answer, getDirection(), up);
    answer.Normalize();
    return answer; }
  Vec3f getScreenUp() const {
    Vec3f answer;
    Vec3f::Cross3(answer, getHorizontal(), getDirection());
    return answer; }
  Vec3f getDirection() const {
    Vec3f answer = point_of_interest - camera_position;
    answer.Normalize();
    return answer; }

  // REPRESENTATION
  Vec3f point_of_interest;
  Vec3f camera_position;
  Vec3f up;
  int width;
  int height;
};

// ====================================================================

class OrthographicCamera : public Camera {

public:
  // CONSTRUCTOR & DESTRUCTOR
  OrthographicCamera(const Vec3f &c, const Vec3f &poi, const Vec3f &u, float s);

  // GL NAVIGATION
  void glInit(int w, int h);
  void zoomCamera(float factor);
  friend std::ostream& operator<<(std::ostream &ostr, const OrthographicCamera &c);

private:
  OrthographicCamera() { assert(0); } // don't use

  // REPRESENTATION
  float size;
  Vec3f lowerLeft;
  Vec3f xAxis;
  Vec3f yAxis;
};

// ====================================================================

class PerspectiveCamera : public Camera {

public:
  // CONSTRUCTOR & DESTRUCTOR
  PerspectiveCamera(const Vec3f &c, const Vec3f &poi, const Vec3f &u, float a);

  // GL NAVIGATION
  void glInit(int w, int h);
  void zoomCamera(float dist);
  friend std::ostream& operator<<(std::ostream &ostr, const PerspectiveCamera &c);
  
private:
  PerspectiveCamera() { assert(0); } // don't use

  // REPRESENTATION
  float angle;
};

// ====================================================================

#endif

